// Copyright 2016 Proyectos y Sistemas de Mantenimiento SL (eProsima).
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

/*!
 * @file IPCSubscriber.hpp
 * This header file contains the declaration of the subscriber functions.
 *
 * This file was generated by the tool fastddsgen.
 */

#ifndef FAST_DDS_GENERATED__IPCSubscriber_HPP
#define FAST_DDS_GENERATED__IPCSubscriber_HPP

#include <condition_variable>

#include <fastdds/dds/domain/DomainParticipant.hpp>
#include <fastdds/dds/domain/DomainParticipantFactory.hpp>
#include <fastdds/dds/subscriber/DataReaderListener.hpp>
#include <fastdds/dds/topic/TypeSupport.hpp>
#include <string>
#include "test.hpp"
#include "IPublisher.h"

#include <fastdds/dds/core/status/SubscriptionMatchedStatus.hpp>
#include <fastdds/dds/domain/DomainParticipantFactory.hpp>
#include <fastdds/dds/subscriber/DataReader.hpp>
#include <fastdds/dds/subscriber/qos/DataReaderQos.hpp>
#include <fastdds/dds/subscriber/qos/SubscriberQos.hpp>
#include <fastdds/dds/subscriber/SampleInfo.hpp>
#include <fastdds/dds/subscriber/Subscriber.hpp>

#include "testPubSubTypes.hpp"
#include <type_traits>


using namespace eprosima::fastdds::dds;

class IPCSubscriberBase {
public:
    virtual ~IPCSubscriberBase() = default;

    virtual void stop() = 0;
};


template<typename MessageT>
class IPCSubscriber : public IPCSubscriberBase,
        public eprosima::fastdds::dds::DataReaderListener
{
public:
    //static_assert(has_register_type<MessageSubType>::value, "MessageSubType must have a register_type member function that takes a DomainParticipant");
    // 订阅者数据接收回调
    typedef std::function<void(const MessageT& sample)> IPCSubscriberDataCallBack;

    //std::unique_ptr<typename MakePubSubType<MessageT>::Type> messageSubType_;
    

    IPCSubscriber(const int& domain_id, const std::string& topicName): factory_(nullptr)
        , participant_(nullptr)
        , subscriber_(nullptr)
        , topic_(nullptr)
        , reader_(nullptr)
        , type_(new typename MakePubSubType<MessageT>::Type)
        //, type_(new OneMessagePubSubType)
        , samples_received_(0)
        , stop_(false)
        ,topicName_(topicName)
    {
        // Create the participant
        DomainParticipantQos pqos = PARTICIPANT_QOS_DEFAULT;
        pqos.name("OneMessage_sub_participant");
        factory_ = DomainParticipantFactory::get_shared_instance();
        participant_ = factory_->create_participant(domain_id, pqos, nullptr, StatusMask::none());
        if (participant_ == nullptr)
        {
            throw std::runtime_error("HelloSecurity Participant initialization failed");
        }

        // Register the type
        // MessageSubType* aa = new MessageSubType;
        // type_ = *((eprosima::fastdds::dds::TypeSupport*)aa);
        type_.register_type(participant_);

        // Create the subscriber
        SubscriberQos sub_qos = SUBSCRIBER_QOS_DEFAULT;
        participant_->get_default_subscriber_qos(sub_qos);
        subscriber_ = participant_->create_subscriber(sub_qos, nullptr, StatusMask::none());
        if (subscriber_ == nullptr)
        {
            throw std::runtime_error("HelloSecurity Subscriber initialization failed");
        }

        // Create the topic
        TopicQos topic_qos = TOPIC_QOS_DEFAULT;
        participant_->get_default_topic_qos(topic_qos);
        topic_ = participant_->create_topic(topicName_, type_.get_type_name(), topic_qos);
        if (topic_ == nullptr)
        {
            throw std::runtime_error("HelloSecurity Topic initialization failed");
        }

        // Create the reader
        DataReaderQos reader_qos = DATAREADER_QOS_DEFAULT;
        subscriber_->get_default_datareader_qos(reader_qos);
        reader_qos.reliability().kind = ReliabilityQosPolicyKind::RELIABLE_RELIABILITY_QOS;
        reader_qos.durability().kind = DurabilityQosPolicyKind::TRANSIENT_LOCAL_DURABILITY_QOS;
        reader_qos.history().kind = HistoryQosPolicyKind::KEEP_ALL_HISTORY_QOS;
        reader_ = subscriber_->create_datareader(topic_, reader_qos, this, StatusMask::all());
        if (reader_ == nullptr)
        {
            throw std::runtime_error("HelloSecurity DataReader initialization failed");
        }
    }


    virtual ~IPCSubscriber() {
        if (nullptr != participant_)
        {
            // Delete DDS entities contained within the DomainParticipant
            participant_->delete_contained_entities();

            // Delete DomainParticipant
            factory_->delete_participant(participant_);
        }
    }

    //! Subscription callback
    void on_data_available(
            eprosima::fastdds::dds::DataReader* reader) override {
        MessageT sample_;
        SampleInfo info;
        while ((!is_stopped()) && (RETCODE_OK == reader->take_next_sample(&sample_, &info)))
        {
            if ((info.instance_state == ALIVE_INSTANCE_STATE) && info.valid_data)
            {
                //std::cout << "Sample '" << std::to_string(++samples_received_) << "  RECEIVED " << sample_.securityData() << std::endl;
                std::cout << "Sample '" << std::to_string(++samples_received_) << "  RECEIVED " << std::endl; 
                if(m_callBack != nullptr) {
                    m_callBack(sample_);
                }
            }
        }
    }

    //! Subscriber matched method
    void on_subscription_matched(
            eprosima::fastdds::dds::DataReader* reader,
            const eprosima::fastdds::dds::SubscriptionMatchedStatus& info) override {
        if (info.current_count_change == 1)
        {
            std::cout << "HelloSecurity Subscriber matched." << std::endl;
        }
        else if (info.current_count_change == -1)
        {
            std::cout << "HelloSecurity Subscriber unmatched." << std::endl;
        }
        else
        {
            std::cout << info.current_count_change
                    << " is not a valid value for SubscriptionMatchedStatus current count change" << std::endl;
        }
    }

    //! Run subscriber
    void run(){
        std::unique_lock<std::mutex> lck(terminate_cv_mtx_);
        terminate_cv_.wait(lck, [this]
                {
                    return is_stopped();
                });
    }

    //! Trigger the end of execution
    void stop() {
        stop_.store(true);
        terminate_cv_.notify_all();
    }

    void setDataHandler(IPCSubscriberDataCallBack callBack) {
        m_callBack = callBack;
    }

    void startListen();

    std::string getTopic() const {
        return topicName_;
    }

private:
    IPCSubscriberDataCallBack m_callBack = nullptr;

    //! Return the current state of execution
    bool is_stopped() {
        return stop_.load();
    }

    std::shared_ptr<eprosima::fastdds::dds::DomainParticipantFactory> factory_;
    eprosima::fastdds::dds::DomainParticipant* participant_;
    eprosima::fastdds::dds::Subscriber* subscriber_;
    eprosima::fastdds::dds::Topic* topic_;
    eprosima::fastdds::dds::DataReader* reader_;
    eprosima::fastdds::dds::TypeSupport type_;
    //MessageSubType type_;
    uint16_t samples_received_;
    std::atomic<bool> stop_;
    mutable std::mutex terminate_cv_mtx_;
    std::condition_variable terminate_cv_;
    std::string topicName_;
};

#endif // FAST_DDS_GENERATED__IPCSubscriber_HPP